<?php

if (!defined('_PS_VERSION_'))
	exit;

class Buybox extends Module
{
	const DB_PREFIX = 'buybox_';
	const API_URL = 'https://api-lv.100shoppers.com';
	const HELP_URL = 'https://help.100shoppers.com/en/articles/4476483-prestashop-plugin';
	const SCRIPT_URL = 'https://shop-lv.100shoppers.com/js/bb-shop.min.js?v=20210517';

	public function __construct()
	{
		$this->name = 'buybox';
		$this->tab = 'advertising_marketing';
		$this->version = '1.2.2';
		$this->author = 'Simply4net';
		$this->bootstrap = true;
		$this->need_instance = 0;
		$this->ps_versions_compliancy = array('min' => '1.6.0');

		parent::__construct();

		$this->displayName = 'BUY.BOX';
		$this->description = $this->l('A module that integrates the online store with BUY.BOX service');

		$this->confirmUninstall = $this->l('Are you sure you want to uninstall BUY.BOX module?');

		if(!Configuration::get('bb-shop-id'))
		{
			$this->warning=$this->l('You did not enter your Shop ID! ');
		}
	}

	public function install()
	{
		if (!parent::install() OR !$this->registerHook('header') OR !$this->registerHook('orderConfirmation') OR
				!$this->registerHook('productTabContent') OR !$this->registerHook('displayFooterProduct') OR
				!$this->CreateCategoryCommissionTable() OR !$this->CreateTransactionsTable()){
			return false;
		}

			return true;
	}

	public function uninstall()
	{
		if (!$this->DropCategoryCommissionTable() OR !$this->DropTransactionsTable() OR !Configuration::deleteByName('bb-shop-id') OR !Configuration::deleteByName('bb-shop-apikey') OR !Configuration::deleteByName('bb-shop-apisecret') OR !parent::uninstall()){
    		return false;
		}

		return true;
	}

	private function CreateCategoryCommissionTable()
	{
		$sql='
        	CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.self::DB_PREFIX.'commissions'.'` (
            `Id` int(11) NOT NULL AUTO_INCREMENT,
			`id_category_shop` INT(11),
			`id_commission` INT(11),
            PRIMARY KEY (`Id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8';

		if(!Db::getInstance()->execute($sql)) return false;
		return true;
	}

	private function CreateTransactionsTable()
	{
		$sql='
        	CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.self::DB_PREFIX.'transactions'.'` (
            `Id` int(11) NOT NULL AUTO_INCREMENT,
			`id_transaction` varchar(25),
			`amount` VARCHAR(25),
			`date` VARCHAR(25),
            PRIMARY KEY (`Id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8';

		if(!Db::getInstance()->execute($sql)) return false;
		return true;
	}

	private function DropCategoryCommissionTable()
	{
		$sql='
        	DROP TABLE `'._DB_PREFIX_.self::DB_PREFIX.'commissions'.'`';

		if(!Db::getInstance()->execute($sql)) return false;
		return true;
	}

	private function DropTransactionsTable()
	{
		$sql='
        	DROP TABLE `'._DB_PREFIX_.self::DB_PREFIX.'transactions'.'`';

		if(!Db::getInstance()->execute($sql)) return false;
		return true;
	}

	public function hookDisplayHeader()
	{
		if (empty(Configuration::get('bb-shop-id'))) {
			return '';
		}

		$output='
<script type="text/javascript">
"use strict";(function(scriptUrl,campaignId){if(!window.bb){window.bb=function(){
for(var _len=arguments.length,params=Array(_len),_key=0;_key<_len;_key++){
params[_key]=arguments[_key]}return window.bb.q.push(params)};window.bb.q=[]
;window.bb.campaignId=campaignId;var script=document.createElement("script")
;var firstScript=document.getElementsByTagName("script")[0];script.async=true
;script.src=scriptUrl;firstScript.parentNode.insertBefore(script,firstScript)}
})("'.self::SCRIPT_URL.'", '.Configuration::get('bb-shop-id').',0);
</script>';

		return $output;
	}

	public function hookProductTabContent($params)
	{
		$version_check_result = (
			version_compare(_PS_VERSION_, '1.7', '<') ||
			(
				isset($params['no_version_check']) &&
				$params['no_version_check']
			)
		);

		if ($version_check_result) {
			if (empty(Configuration::get('bb-shop-id'))) {
				return '';
			}

			$product=$params['product'];

			if(Validate::isLoadedObject($product))
			{
				$output='<script type="text/javascript">bb("productView", { productId: "'.$product->id.'" });</script>';

				return $output;
			}
		}
	}

	public function hookDisplayFooterProduct($params)
	{
		if (version_compare(_PS_VERSION_, '1.7', '>=')) {
			$product = new Product(
				(int)Tools::getValue('id_product'),
				$this->context->language->id,
				Shop::getContextShopID()
			);

			$params = array(
				'product' => $product,
				'no_version_check' => true
			);

			return $this->hookProductTabContent($params);
		}
	}

	public function hookOrderConfirmation($params)
	{
		if (empty(Configuration::get('bb-shop-id'))) {
			return '';
		}

		if (isset($params['objOrder'])) {
			$order = $params['objOrder'];
		} else {
			$order = $params['order'];
		}

		if(Validate::isLoadedObject($order))
		{
			$products = $order->getProducts();
			$currency=Currency::getCurrencyInstance($order->id_currency);
			$carrier=new Carrier($order->id_carrier);

			$list_products='';

			foreach($products as $product)
			{
				$id_commission=$this->getCommission($product['id_category_default']);

				$list_products.='
{ "productId":"'.$product['product_id'].'", "quantity": '.$product['product_quantity'].', "commissionId": '.$id_commission.', "net": '.$product['product_price'].' , "gross": '.$product['product_price_wt'].', "currency": "'.$currency->iso_code.'" }';

				if($product!=end($products)){
					$list_products.=',';
				}
			}

			$output='<script type="text/javascript">
bb("order", { "orderId": "'.$order->reference.'", "products": ['.$list_products.'],
"delivery": {"cost": '.$order->total_shipping.', "type": "", "partner": "'.$carrier->name.'"},
"payment": {"mode": "", "method": "'.$order->payment.'"}
});
</script>';

			/* Start: Weryfikacja transakcji/zamówienia przez API */
			if (!(empty(Configuration::get('bb-shop-apikey')) || empty(Configuration::get('bb-shop-apisecret')))) {

				$this->addOrderToBuffer($order->reference, $params['total_to_pay'], $order->date_add);

				$order_to_accept=array();
				$order_to_reject=array();

				foreach($this->getBufferOrders() as $buffer_order)
				{
					if($this->getStatusOrder($buffer_order['id_transaction'])==5)
					{
						array_push($order_to_accept, $buffer_order['id_transaction']);
					}
					else if($this->getStatusOrder($buffer_order['id_transaction'])==6 || $this->getStatusOrder($buffer_order['id_transaction'])==7)
					{
						array_push($order_to_reject, $buffer_order['id_transaction']);
					}
				}

				$result_accept=$this->AcceptManyOrdersApi($order_to_accept);
				$result_reject=$this->RejectManyOrdersApi($order_to_reject);

				$this->removeOrdersFromBuffer($result_accept, $result_reject);
			}

			/* End: Weryfikacja transakcji/zamówienia przez API */

			return $output;
		}

	}

	private function getParent($id)
	{
		$id_parent=DB::getInstance()->getValue('select id_parent from '._DB_PREFIX_.'category where id_category='.intval($id));

		return $id_parent;
	}

	private function getCommissionById($id)
	{
		$db=DB::getInstance();

		if($db->getValue('select count(*) from '._DB_PREFIX_.self::DB_PREFIX.'commissions;')>0)
		{
			$id_commission=$db->getValue('select id_commission from '._DB_PREFIX_.self::DB_PREFIX.'commissions where id_category_shop='.intval($id));
			if($id_commission>0){
				return  $id_commission;
			}
			else {
				return null;
			}
		}
		else {
			return null;
		}
	}

	private function getTree($category, $string)
	{
		$string.='<tr><td style="width:50%;"><p style="padding-left: 0px">'.$this->l("All categories").'</p></td><td style="width: 50%;"><input type="text" style="width: 100px;" id="category2" name="category2" value="'.$this->getCommissionById(2).'" /></td></tr>';

		foreach($category as $cat)
		{
			$string.='<tr><td style="width:50%;"><p style="padding-left: 20px">'.$cat['name'].'</p></td><td style="width: 50%;"><input type="text" style="width: 100px;" id="category'.$cat['id'].'" name="category'.$cat['id'].'" value="'.$this->getCommissionById($cat['id']).'" /></td></tr>';

			if(sizeof($cat['children'])>0)
			{
				foreach($cat['children'] as $ca){
					$string.='<tr><td style="width:50%;"><p style="padding-left: 40px">'.$ca['name'].'</p></td><td style="width: 50%;"><input type="text" style="width: 100px;" id="category'.$ca['id'].'" name="category'.$ca['id'].'" value="'.$this->getCommissionById($ca['id']).'" /></td></tr>';

					if(sizeof($ca['children'])>0){
						foreach($ca['children'] as $c){
							$string.='<tr><td style="width:50%;"><p style="padding-left: 60px">'.$c['name'].'</p></td><td style="width: 50%;"><input type="text" style="width: 100px;" id="category'.$c['id'].'" name="category'.$c['id'].'" value="'.$this->getCommissionById($c['id']).'" /></td></tr>';

							if(sizeof($c['children'])>0){
								foreach($c['children'] as $a){
									$string.='<tr><td style="width:50%;"><p style="padding-left: 80px">'.$a['name'].'</p></td><td style="width: 50%;"><input type="text" style="width: 100px;" id="category'.$a['id'].'" name="category'.$a['id'].'" value="'.$this->getCommissionById($a['id']).'" /></td></tr>';

									if(sizeof($a['children'])>0){
										foreach($a['children'] as $b){
											$string.='<tr><td style="width:50%;"><p style="padding-left: 100px">'.$b['name'].'</p></td><td style="width: 50%;"><input type="text" style="width: 100px;" id="category'.$b['id'].'" name="category'.$b['id'].'" value="'.$this->getCommissionById($b['id']).'" /></td></tr>';
										}
									}
								}
							}

						}
					}
				}
			}
		}

		return $string;
	}

	private function getCommission($id_category)
	{
		$id_commission=DB::getInstance()->getValue('select id_commission from '._DB_PREFIX_.self::DB_PREFIX.'commissions where id_category_shop='.$id_category);

		if($id_commission==0)
		{
			return $this->getCommission($this->getParent($id_category));
		}

		return $id_commission;
	}

	private function GetListWaitingOrdersApi()
	{
		if (empty(Configuration::get('bb-shop-apikey')) || empty(Configuration::get('bb-shop-apisecret'))) {
			return;
		}

		$url=self::API_URL.'/api/transactions';
		$signature=hash_hmac('sha1',utf8_encode('GET /api/transactions\n'), Configuration::get('bb-shop-apisecret'));
		$header='X-BB-API-Authorization: '.Configuration::get('bb-shop-apikey').':'.$signature;

		$options=array(
				'http' => array(
						'header'=>$header,
						'method'=>'GET',
						'content'=>''
				)
		);

		$context=stream_context_create($options);
		$result=file_get_contents($url,false, $context);

		return $result;
	}

	private function AcceptOneOrderApi($number_order)
	{
		if (empty(Configuration::get('bb-shop-apikey')) || empty(Configuration::get('bb-shop-apisecret'))) {
			return;
		}

		$url=self::API_URL.'/api/transactions/'.$number_order.'/accept';
		$signature=hash_hmac('sha1',utf8_encode('POST /api/transactions/'.$number_order.'/accept\n'), Configuration::get('bb-shop-apisecret'));
		$header='X-BB-API-Authorization: '.Configuration::get('bb-shop-apikey').':'.$signature;

		$options=array(
				'http' => array(
						'header'=>$header,
						'method'=>'POST',
						'content'=>''
				)
		);

		$context=stream_context_create($options);
		$result=file_get_contents($url,false, $context);

		return $result;
	}

	private function AcceptManyOrdersApi($array_number_orders)
	{
		if (empty(Configuration::get('bb-shop-apikey')) || empty(Configuration::get('bb-shop-apisecret'))) {
			return;
		}

		$url=self::API_URL.'/api/transactions/accept';
		$signature=hash_hmac('sha1',utf8_encode('POST /api/transactions/accept\n'.$array_number_orders), Configuration::get('bb-shop-apisecret'));
		$header='X-BB-API-Authorization: '.Configuration::get('bb-shop-apikey').':'.$signature;

		$options=array(
				'http' => array(
						'header'=>$header,
						'method'=>'POST',
						'content'=>$array_number_orders
				)
		);

		$context=stream_context_create($options);
		$result=file_get_contents($url,false, $context);

		return $result;
	}

	private function RejectOneOrderApi($number_order)
	{
		if (empty(Configuration::get('bb-shop-apikey')) || empty(Configuration::get('bb-shop-apisecret'))) {
			return;
		}

		$url=self::API_URL.'/api/transactions/'.$number_order.'/reject';
		$signature=hash_hmac('sha1',utf8_encode('POST /api/transactions/'.$number_order.'/reject\n'), Configuration::get('bb-shop-apisecret'));
		$header='X-BB-API-Authorization: '.Configuration::get('bb-shop-apikey').':'.$signature;

		$options=array(
				'http' => array(
						'header'=>$header,
						'method'=>'POST',
						'content'=>''
				)
		);

		$context=stream_context_create($options);
		$result=file_get_contents($url,false, $context);

		return $result;
	}

	private function RejectManyOrdersApi($array_number_orders)
	{
		if (empty(Configuration::get('bb-shop-apikey')) || empty(Configuration::get('bb-shop-apisecret'))) {
			return;
		}

		$url=self::API_URL.'/api/transactions/reject';
		$signature=hash_hmac('sha1',utf8_encode('POST /api/transactions/reject\n'.$array_number_orders), Configuration::get('bb-shop-apisecret'));
		$header='X-BB-API-Authorization: '.Configuration::get('bb-shop-apikey').':'.$signature;

		$options=array(
				'http' => array(
						'header'=>$header,
						'method'=>'POST',
						'content'=>$array_number_orders
				)
		);

		$context=stream_context_create($options);
		$result=file_get_contents($url,false, $context);

		return $result;
	}

	private function getBufferOrders()
	{
		$orders=Db::getInstance()->ExecuteS('select * from '._DB_PREFIX_.self::DB_PREFIX.'transactions');

		return $orders;
	}

	private function addOrderToBuffer($id_transaction, $amount, $date)
	{
		Db::getInstance()->insert(self::DB_PREFIX.'transactions', array(
				'id_transaction'=>$id_transaction,
				'amount'=>$amount,
				'date'=>$date
		));
	}

	private function removeOrdersFromBuffer($result_accept, $result_reject)
	{
		if(property_exists(json_decode($result_accept), 'accepted')==true)
		{
			foreach(json_decode($result_accept)->accepted as $order)
			{
				Db::getInstance()->delete(self::DB_PREFIX.'transactions', 'id_transaction=='.$order->id);
			}
		}

		if(property_exists(json_decode($result_reject), 'rejected')==true)
		{
			foreach(json_decode($result_reject)->rejected as $order)
			{
				Db::getInstance()->delete(self::DB_PREFIX.'transactions', 'id_transaction=='.$order->id);
			}
		}

	}

	private function getStatusOrder($reference_order)
	{
		$id_order=Db::getInstance()->getValue('select id_order from '._DB_PREFIX_.'orders where reference="'.$reference_order.'";');
		$order=new Order($id_order);


		return $order->getCurrentState();
	}

	public function getContent()
	{

		if(Tools::isSubmit('submitGetBuyBox'))
		{
			Configuration::updateValue('bb-shop-id', Tools::getValue('bb-shop-id'));
			Configuration::updateValue('bb-shop-apikey', Tools::getValue('bb-shop-apikey'));
			Configuration::updateValue('bb-shop-apisecret', Tools::getValue('bb-shop-apisecret'));
		}

		if(Tools::isSubmit('submitCategoryGetBuyBox'))
		{
			DB::getInstance()->delete(self::DB_PREFIX.'commissions');

			foreach($_POST as $key => $value)
			{
				if(strpos($key,'category')===0)
				{
					DB::getInstance()->insert(self::DB_PREFIX.'commissions',[
							'id_category_shop'=>str_replace('category','',$key),
							'id_commission'=>$value
					]);
				}
			}
		}

		$category=new Category(Configuration::get('PS_ROOT_CATEGORY'));
		$tree=$category->recurseLiteCategTree(100);

		$string='<table style="width: 100%;">';
		$form_category=$this->getTree($tree['children'][0]['children'], $string);

		$this->context->smarty->assign(array(
			'action'=>$_SERVER['REQUEST_URI'],
			'bb_shop_id'=>Configuration::get('bb-shop-id'),
			'bb_shop_apikey' =>Configuration::get('bb-shop-apikey'),
			'bb_shop_apisecret'=>Configuration::get('bb-shop-apisecret'),
			'category_shop'=>$form_category.'</table>',
			'export_link' => $this->context->link->getModuleLink('buybox', 'export'),
			'help_url' => self::HELP_URL,
        ));

		return $this->display(__FILE__, 'views/templates/admin/admin_buybox.tpl');
	}
}
