<?php

class buyboxexportModuleFrontController extends ModuleFrontController
{
    public function display()
    {
        $link = $this->context->link;
        $xml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>
            <offers xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" version=\"1\">";

        $products = Product::getProducts($this->context->language->id, 0, 0, 'id_product', 'ASC');
        foreach ($products as $product) {
            $product['calculated_price'] = Product::getPriceStatic($product['id_product']);
            $product['calculated_quantity'] = Product::getQuantity($product['id_product']);
            if (!boolval($product['active'])) continue;
            $product['category_path'] = $this->getCategoryPathString($product['id_category_default']);
            $product['desc'] = substr($product['description'], 0, 30000); //30 000 is maximum length of description
            $product['link'] = $link->getProductLink($product);
            $xml .= "\n".$this->prepareProductXml($product);
        }

        $xml .= "\n</offers>";
        header('Content-type: application/xml');
        echo $xml;
    }

    private function prepareProductXml(array $product)
    {
            $xml = '<o id="' . $product['id_product'] . '" url="' . htmlentities($product['link']) . '" price="' . $product['calculated_price'] . '" stock="' . $product['calculated_quantity'] . '">';

            $xml .= $this->prepareXmlBlock("cat", $product['category_path']);
            $xml .= $this->prepareXmlBlock("name", $product['name']);
            $xml .= $this->prepareXmlBlock("desc", $product['desc']);

            $manufacturer = $this->prepareXmlBlock("a", $product['manufacturer_name'], "Producent");
            $manufacturerCode = $this->prepareXmlBlock("a", "", "Kod_producenta");
            $ean = $this->prepareXmlBlock("a", $product['ean13'], "EAN");
            $attributes = $manufacturer . $manufacturerCode . $ean;

            $xml .= $this->prepareXmlBlock("attrs", $attributes, null, false);

            $xml .= "</o>";

        return $xml;
    }

    /**
     * @param $categoryId int
     * @return string
     */
    private function getCategoryPathString($categoryId)
    {
        $category = new Category($categoryId);
        $name = $category->name[$this->context->language->id];
        if (!$category->is_root_category && $category->id_parent) {
            return $this->getCategoryPathString($category->id_parent) . "/" . $name;
        }
        return $name;
    }

    private function prepareXmlBlock($tag, $content, $name = null, $isCData = true)
    {
        $nameAttribute = !$name ? "" : " name=\"" . $name . "\"";
        $block = "<" . $tag . $nameAttribute . ">";
        if ($isCData) {
            $block .= "<![CDATA[";
        }
        $block .= $content;
        if ($isCData) {
            $block .= "]]>";
        }
        $block .= "</" . $tag . ">";
        return $block;
    }
}